from django.shortcuts import render
from django.utils import timezone
import datetime
from django.db.models import Count
from symonitoring_model.models import Alert
from symonitoring_model.models import Host, Hold, HoldInform
from symonitoring_model.models import Hostserviceinstance
from symonitoring_model.models import Nagioslog as nl
from symonitoring_simulator.helpers import next_problemid
from django.template.defaulttags import register
from django.conf import settings
from django.db.models import Q


def consts_to_dict(object_):
    """Converts a constants object to a dictionary"""
    new = {}
    for const in dir(object_):
        if not const.startswith("_"):
            new[getattr(object_, const)] = const
    return new


class NagiosLog(object):
    """Constants for nagios log"""
    STATE_OPEN             = 0
    STATE_ALERTED          = 1
    STATE_NO_SLA           = 2
    STATE_MALFORMED_ROW    = 3
    STATE_HOST_DISABLED    = 4
    STATE_ALERT_CLOSED     = 5
    STATE_ALERT_REPETITION = 6
    STATE_NO_TEAM          = 7
    STATE_IN_MAINTENANCE   = 8
    STATE_SOFT             = 9
    STATE_HOST_DOWN        = 10
    STATE_NO_PREVIOUS_DOWN = 11
    STATE_DEPENDENCY_DOWN  = 12
    STATE_DOES_NOT_EXIST   = 13
    STATE_WARNING          = 14
    STATE_GLOBAL_DISABLE   = 15


class AlertState(object):
    """Constants for alert states"""
    STATE_UP          = 'UP'
    STATE_DOWN        = 'DOWN'
    STATE_UNREACHABLE = 'UNREACHABLE'
    STATE_OK          = 'OK'
    STATE_WARNING     = 'WARNING'
    STATE_CRITICAL    = 'CRITICAL'


alert_state_dict = consts_to_dict(AlertState)
nagios_log_dict = consts_to_dict(NagiosLog)


@register.filter
def get_log_state(key):
        if key:
            return nagios_log_dict[int(key)]
        return None


@register.filter
def get_alert_state(key):
        if key:
            return alert_state_dict[key]
        return None


def simulator_view(request):
    hosts       = Host.objects.filter(dependencyid=None)
    states      = {
        #                 State for host         State for service
        'UP/OK':         (nl.STATE_UP          , nl.STATE_OK),
        'DOWN/CRITICAL': (nl.STATE_DOWN        , nl.STATE_CRITICAL),
        'UNREACHABLE':   (nl.STATE_UNREACHABLE , None),
        'WARNING':       (None                 , nl.STATE_WARNING),
        'UNKNOWN':       (None                 , nl.STATE_UNKNOWN),
    }
    state_types = sorted(nl.CHOICES_STATE_TYPE, key=lambda x: x[0])
    alerts = Alert.objects.all().filter(
        Q(state=AlertState.STATE_DOWN) | Q(state=AlertState.STATE_CRITICAL),
        acknowledged=False
    )
    holds = Hold.objects.all()
    hold_informs = HoldInform.objects.all()
    logs = nl.objects.filter(
        timestamp__gt=timezone.now() - datetime.timedelta(
            minutes=60
        )
    ).values('processed').annotate(
        num_proc=Count('id')
    )

    if request.POST:
        state_tuple = states[request.POST.get('state')]
        statetype   = request.POST.get('state_type')

        for key, value in request.POST.iteritems():
            try:
                t, i = key.split('-')
            except:
                continue

            if t == 'host':
                objecttype = nl.OBJECT_TYPE_HOST
                state      = state_tuple[0]
                if not state:
                    continue

                host = Host.objects.get(id=i)

                name         = host.hostname
                hostname     = host.hostname
                checkcommand = 'check-host-alive'
                address      = host.ipv4
                output       = 'PING CRITICAL'

            elif t == 'hostserviceinstance':
                objecttype = nl.OBJECT_TYPE_SERVICE
                state      = state_tuple[1]
                if not state:
                    continue

                hostserviceinstance = Hostserviceinstance.objects.get(id=i)

                name         = '%s__%s' % (
                    hostserviceinstance.hostserviceid.serviceid.name,
                    hostserviceinstance.name
                )
                hostname     = hostserviceinstance.hostserviceid.hostid.hostname
                checkcommand = hostserviceinstance.hostserviceid.serviceid.command
                address      = hostserviceinstance.hostserviceid.hostid.ipv4
                output       = 'Connection refused'

            else:
                continue

            entry = nl(
                objecttype    = objecttype,
                statetype     = statetype,
                state         = state,
                name          = name,
                hostname      = hostname,
                checkcommand  = checkcommand,
                address       = address,
                output        = output,
                perfdata      = None,
                duration      = '0d 0h 0m 0s',
                datetime      = datetime.datetime.now(),
                attempt       = 1,
                processed     = 0,
                problemid     = next_problemid(),
                lastproblemid = 0,
                alert         = None
            )
            entry.save()


    return render(
        request,
        'symonitoring_simulator/index.html',
        {
            'hosts'               : hosts,
            'states'              : states,
            'state_types'         : state_types,
            'alerts'              : alerts,
            'holds'               : holds,
            'hold_informs'        : hold_informs,
            'logs'                : logs,
            'BOSH_SERVICE_URL'    : settings.BOSH_SERVICE_URL,
            'DEFAULT_JABBER_USER' : settings.DEFAULT_JABBER_USER,
        }
    )
